
/*
 * Copyright (c) 1992 Kent State University.
 * All rights reserved.  The Kent State University 
 * software License Agreement specifies the terms and conditions 
 * for redistribution.
 *
 *	@(#)signalf.c	1.1 (Kent State) 2/28/92
 *
 *	Author: Paul B. Ricks
 *
 *	The spirit of the Gnu Licensing Agreement Applies to this software and 
 *	Kent State University should be substituted where ever Gnu appears.
 *
 */

/*
 *	signalf.c  -- the routines in this file depend on a volatile variable <abort_task>.
 *	since this variable had to be declared static, all functions that direcly reference
 *	it have to be declared within the same file. This might not be true for other systems,
 *	but was for ours.  Therefore, <abort_task> was never being set until I did what was
 *	mentioned above.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright (c) 1992 Kent State University.\n\
 All rights reserved.\n";
#endif /* not lint */

#ifndef lint
static char sccsid[] = "@(#)signalf.c 1.1 (Kent State Univ.) 2/28/92";
#endif /* not lint */
/*

                            NO WARRANTY

   BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

                     END OF TERMS AND CONDITIONS
*/


#include "signalf.h"

#define	SELECT_SECONDS	8
#define	CONNECT_SECONDS	10
#define XCOMMD_PORT	5009

int cc;
extern char MESSAGE_IN[];
extern char MESSAGE_OUT[];
static int abort_task;

extern 	int xcmmd_aplen;
extern 	struct sockaddr *xcmmd_ap;
extern 	struct sockaddr *relay_ap;
extern 	struct sockaddr_in xcmmd_addr;
static void sigalrm_handler(), sigusr1_handler(), sigusr2_handler(), sigterm_handler(), sigpipe_handler();

static struct itimerval itimer;
static struct itimerval ztimer;

int	initEnvironment(argc, argv)
	int argc;
	char **argv;
{
	close(filed_con[0]);
	register_connection(argc, argv);

	dup2(fds[1], 0);
	close(fds[0]); 
	close(fds[1]);

	dup2(relay_sockt, 1);
	close(relay_sockt);  

	pto_child_sockt = 0;	
	relay_sockt = 1;
	errno = 0;
	signal(SIGINT, (void (*)())sigterm_handler);	
	signal(SIGALRM, (void (*)())sigalrm_handler);
	signal(SIGUSR1, (void (*)())sigusr1_handler); /* enable caller to cancel a call */	
	signal(SIGUSR2, (void (*)())sigusr2_handler); /* enable caller to abort in the middle of a call */	
	signal(SIGTERM, (void (*)())sigterm_handler);	
	signal(SIGPIPE, (void (*)())sigpipe_handler);	

	pc_msglen = sizeof(LOCAL_MSG);
}

int	processLRequest()
{
	static LOCAL_MSG lmsg;

	cc = read(0, (char *)&lmsg, pc_msglen);
	if (cc < pc_msglen)
	    return (0);

	switch (lmsg.type) {
	   case	MAKE_CALL:	make_call(lmsg.line);
				break;
	   case ACCEPT_CALL:	accept_call(lmsg.id);
				break;
	   case PHONELINE:      toggle_phoneline();
				break;
	   case AUTO_ANSWER:    answer_machine =  answer_machine ?  0:1;
				break;
	   case QUIT:		error_exit(0);
	   default:		;
	}
	abort_task = FALSE;
}

int	processDRequest()
{
	static CTL_MSG	rmsg;
	static int rmsglen = sizeof(rmsg);
	static CTL_RESPONSE response;
	static struct sockaddr_in fromaddr;
	static int faplen = sizeof(fromaddr);
	static struct sockaddr *fap = (struct sockaddr *)&fromaddr;

	do {
		cc = recvfrom(relay_sockt, (char *)&rmsg, rmsglen, 0, fap, &faplen);
		if (cc != sizeof(rmsg)) {
		   if (errno == EINTR)
		       continue;
		   break;
		}
	} while (cc < 0);

	if (errno != 0) return (0);

	switch (rmsg.type) {
	   case   TALK_REQUEST:
	   case   ANNOUNCE:  do_announcement(&rmsg);
			     break;
	   case   DELETE:    do_deletion(&rmsg);
			     break;
	   case   RE_REGISTER:
			     msg.type = REGISTER;
			     msg.answer = phoneline_status;
			     msg.ctl_addr = rmsg.addr = *relay_ap;
			     msg_out(relay_sockt, fap, faplen, (char *)&msg, sizeof(msg), &response,
				     (struct sockaddr *)&fromaddr, TRUE);
			     if (response.answer != SUCCESS)
				 relay_msg(URGENT_RESTART, 0);
			     break;
	   case   QUIT:	     error_exit(0);
			     break;
	   default:	     fprintf(stderr, "Got something I don't know:: type (%d)\n", rmsg.type);
	}
	abort_task = FALSE;
}

int	do_announcement(msg)
	CTL_MSG *msg;
{
	int rv,	i;
	int freshcall;
	int xresponse, sockt;
	CTL_RESPONSE reply, resp;
	CTL_RESPONSE *rp = &reply;
	struct sockaddr_in addr;
	struct sockaddr fromaddr;
	struct timeval *apt, cmtime;
	char hmachine_name[MAX_MSGBUF];


	freshcall = isfreshRequest(msg);
	if (callwaiting < MAX_QUE_CALLS) {
	    if (!answer_machine)
	        for (i = 0; i < MAX_QUE_CALLS; i++) 
		     if (que_table[i].id != -1) {
		         if (!strcmp(msg->l_name, que_table[i].call.l_name)) {
			     que_table[i].call = *msg;
			     return (0);
		         }	
		     }
	    if (!freshcall) return(0);
	    reply.vers = msg->vers;
	    reply.type = msg->type;
	    reply.answer = SUCCESS;
	    reply.addr = msg->ctl_addr;
	    reply.xcomm = XCOMMFLAG;

	    if (!answer_machine && callwaiting < MAX_QUE_CALLS)
		xresponse = relay_request(msg, hmachine_name);
	    else if (!answer_machine)
		xresponse = -1;
	    else
		xresponse = ACCEPT_CALL;

	    if (get_sockt(SOCK_DGRAM, &sockt, &addr)) {
		reply.answer = PERMISSION_DENIED;
		msg_out(relay_sockt, &msg->ctl_addr, sizeof(msg->ctl_addr), (char *)rp,
			sizeof(*rp), &resp, &fromaddr, FALSE);
		relay_msg(RESTART, 0);
		return (0);
	    }

	    if (!abort_task) {
		if (answer_machine || xresponse == ACCEPT_CALL || xresponse == TAKE_MSG)
		    reply.answer = SUCCESS;
		else if (xresponse == PUT_ONHOLD) 
		    reply.answer = ON_HOLD;
		else
		    reply.answer = PERMISSION_DENIED;
	    }
	    else
		reply.answer = PERMISSION_DENIED;
	}
	else {
	    if (!freshcall) return(0);
	    xresponse = reply.answer = (msg->xcomm == XCOMMFLAG) ? TRY_LATER:PERMISSION_DENIED;
	}

	rv = msg_out(sockt, &msg->ctl_addr, sizeof(msg->ctl_addr), (char *)rp, sizeof(*rp), &resp, &fromaddr,
		     FALSE);

	if (rv || abort_task || (xresponse != ACCEPT_CALL && xresponse != TAKE_MSG && 
	    xresponse != PUT_ONHOLD)) {
	    close(sockt);
	    abort_task = 0;
	    if (rv) 
		relay_msg(rv, 0);
	    return (0);
	}

	if (xresponse == PUT_ONHOLD) {
	    for (i = 0; i < MAX_QUE_CALLS; i++)
	         if (que_table[i].id == -1) {
		     que_table[i].call = *msg;
		     que_table[i].id = i;
		     gettimeofday(&que_table[i].arrivaltime, (struct timezone *)0);
		     callwaiting++;
		     relay_msg(WILL_HOLD, i);
		     close(sockt);
		     return (0);
		 }
	    relay_msg(RESTART, 0);
	    close(sockt); /* should never get here */
	    return (0);
	}

	close(sockt);
	if (resp.answer != SUCCESS && !answer_machine)
	    relay_msg(DAEMON_RESPONSE, resp.answer);
	else if (resp.answer == SUCCESS) {
	    if (xresponse == TAKE_MSG)/* can't have xresponse == TAKE_MSG && answer_machine == On */
		answer_machine = TRUE;
	    establish_connection(msg, &resp, &fromaddr);
	    if (xresponse == TAKE_MSG)
		answer_machine = FALSE;
	}

	return (0);
}

int	relay_msg(msg_type, ecode)
	int  msg_type;
	int  ecode;
{

	if (abort_task) {
	    abort_task = FALSE;
	    return (0);
	}

	pc_msg.id = ecode;
	pc_msg.type = msg_type;

	cc = write(0, (char *)&pc_msg, pc_msglen);
	if (cc < pc_msglen)
	    error_exit();
}

int	lookout(s, ns)
	int s;
	int *ns;
{
	int cc;
	fd_set readmask;
	struct timeval timeout;

	FD_ZERO(&readmask);
	FD_SET(s, &readmask);

	timeout.tv_usec = 0;
	timeout.tv_sec = SELECT_SECONDS;

	errno = 0;
	if ((cc = select(s + 1, &readmask, (fd_set *)0, (fd_set *)0, &timeout)) <= 0) {
	    if (errno == EINTR || abort_task) 
		return (abort_task);
	    else if (cc == 0)
		return (-1);
	    return (SELECT_ERROR);
	}
	if (FD_ISSET(s, &readmask)) {
	    if ((*ns = accept(s, (struct sockaddr *)0, (int *)0)) < 0)
		 return (ACCEPT_ERROR);
	    return (0);
	}
	return (-1);
}

int	make_call(line_in)
	char line_in[];
{
	int tresponse;
	int fcount = 0;
	int sockt1, sockt2;
	struct sockaddr *hap;
	int rv, i, nsockt;
	long remote_id, local_id;
	struct sockaddr fromaddr;
	CTL_RESPONSE resp1, resp2;
	struct itimerval itimer, ztimer;
	char errorcode, contact1, contact2, tries;
	struct sockaddr_in addr1, addr2, hisaddr;
	struct in_addr his_machine_addr;

	extern u_short daemon_port;

	if (get_hisinfo(line_in, &his_machine_addr))
	    return (0);

	if ((cc = get_sockt(SOCK_DGRAM, &sockt1, &addr1))) {
	    relay_msg(cc, 0);
	    return (0);
	}
	if ((cc = get_sockt(SOCK_STREAM, &sockt2, &addr2))) {
	    close(sockt1);
	    relay_msg(cc, 0);
	    return (0);
	}
	if (listen(sockt2, 5) < 0) {
	    close(sockt1);
	    close(sockt2);
	    relay_msg(LISTEN_ERROR, 0);
	    return (0);
	}

	msg.ctl_addr = *(struct sockaddr *)&addr1;
	msg.addr = *(struct sockaddr *)&addr2;

	memset((char *)&hisaddr, '\0', sizeof(hisaddr));
	hisaddr.sin_family = AF_INET;

#ifdef XCOMM_NO_SU
	hisaddr.sin_port = htons(XCOMMD_PORT);
#else
	hisaddr.sin_port = htons(daemon_port);
#endif
	hisaddr.sin_addr = his_machine_addr;
	hap = (struct sockaddr *)&hisaddr;
	xcmmd_addr.sin_addr = my_machine_addr;

	remote_id = local_id = 0;
	contact1 = contact2 = FALSE;
	resp1.answer = resp2.answer = FAILED;

	errorcode = 0;
	itimer.it_value.tv_sec = 6;
	itimer.it_value.tv_usec = 0;
	itimer.it_interval = itimer.it_value;

	ztimer.it_value.tv_sec = ztimer.it_value.tv_usec = 0;
	ztimer.it_interval = ztimer.it_interval;

	for (tries = 0; !abort_task && tries < MAX_TRIES * 10; tries++) { /* why 10 ? why not ! */
	     contact1 = FALSE;
	     msg.type = ANNOUNCE;
	     relay_msg(TRYING_REMOTE, 0);

	     rv = msg_out(sockt1, hap, sizeof(*hap), (char *)&msg, sizeof(msg), &resp1, &fromaddr, TRUE);

	     if (resp1.xcomm == XCOMMFLAG || abort_task) {
	         contact1 = TRUE;
		 break;
	     }
	     if (rv || resp1.answer != SUCCESS) {
		 errorcode = rv ? rv:resp1.answer;
		 break;
	     }
	     contact1 = TRUE;

	     remote_id = ntohl(resp1.id_num);
	     msg.id_num = htonl(remote_id);
	     msg.type = LEAVE_INVITE;

	     contact2 = FALSE;
	     relay_msg(TRYING_LOCAL, 0);
	     rv = msg_out(sockt1, xcmmd_ap, xcmmd_aplen, (char *)&msg, sizeof(msg), &resp2, &fromaddr, TRUE);

	     if (rv || resp2.answer != SUCCESS) {
		 errorcode = rv ? rv:resp2.answer;
		 break;
	     }
	    
	     contact2 = TRUE;
	     local_id = ntohl(resp2.id_num);
	     relay_msg(LEFT_REQUEST, 0);

	     if (!(errorcode = lookout(sockt2, &nsockt)))
		 break;
	     else if (errorcode != -1)
		 break;
	    
	     msg.id_num = htonl(resp1.id_num + 1);
	     if (tries + 1 < MAX_TRIES * 10) {
	         relay_msg(RINGING_AGAIN, 0);
	         setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);
		 pause();
	         setitimer(ITIMER_REAL, &ztimer, (struct itimerval *)0);
	     }
	}

	tresponse = resp1.answer;
	if ((resp1.xcomm == XCOMMFLAG && resp1.answer == SUCCESS) || resp1.answer == ON_HOLD) {
	    if (!rv) {
		tries = 0;
		tresponse = 0;
		if (resp1.answer == ON_HOLD)
		    relay_msg(ON_HOLD, 0);
	        for (; !abort_task ; )
		     if (!(errorcode = lookout(sockt2, &nsockt)) && !abort_task)
			 break;
		     else if (errorcode == -1 && resp1.answer != ON_HOLD)
			 relay_msg(HASNT_CONNECTED, 0);
		     else if (errorcode == -1)
			 relay_msg(ON_HOLD, 0);
		     else 
			 break;
	    }
	}
	if (abort_task) errorcode = 0;

	msg.type = DELETE;
	if (contact1) {
	    msg.id_num = htonl(resp1.id_num);
	    msg_out(sockt1, hap, sizeof(*hap), (char *)&msg, sizeof(msg), &resp1, &fromaddr, FALSE);
	}
	if (contact2) {
	    msg.id_num = htonl(resp2.id_num);
	    msg_out(sockt1, xcmmd_ap, xcmmd_aplen, (char *)&msg, sizeof(msg), &resp2, &fromaddr, FALSE);
	}

	if (errorcode || tries >= MAX_TRIES || tresponse != SUCCESS) {
	    if (tries >= MAX_TRIES) 
		relay_msg(TRY_LATER, 0);
	    else if (tresponse != SUCCESS)
		relay_msg(DAEMON_RESPONSE, tresponse);
	    else
		relay_msg(errorcode, 0);
	}
	else if (!abort_task)
	    send_file_d(nsockt);

	close(sockt1);
	close(sockt2);
	return (0);
}

int    	msg_out(sockt, addr_out, addr_len, data, datalen, response, addr_in, waitflag)
	int sockt;
	struct sockaddr *addr_out;
	int addr_len;
	char *data;
	int datalen;
	CTL_RESPONSE *response;
	struct sockaddr *addr_in;
	int waitflag;
{
	char quit;
	char t_abort_task = abort_task;
	fd_set readmask;
	struct timeval timeout;
	struct sockaddr fromaddr;
	int i, fromlen = sizeof(fromaddr);

#define	DWAITING_TIME		8

	abort_task = FALSE;
	response->answer = FAILED;
	itimer.it_value.tv_sec = DWAITING_TIME;
	itimer.it_value.tv_usec = 0;
	itimer.it_interval = itimer.it_value;

	ztimer.it_value.tv_sec = ztimer.it_value.tv_usec = 0;
	ztimer.it_interval = ztimer.it_interval;

	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);
	for (quit = FALSE, i = 0; i < MAX_TRIES && !quit && !abort_task; i++) {
	     cc = sendto(sockt, data, datalen, 0, addr_out, addr_len);
	     if (cc == datalen)
		 quit = TRUE;
	     else if (errno && errno != EINTR)
		 break;

	}
	setitimer(ITIMER_REAL, &ztimer, (struct itimerval *)0);

	if (abort_task)
	    return(TASK_ABORTED);
	abort_task = t_abort_task;

	if (!quit)
	    return(SENDTO_ERROR);

	if (!waitflag) {
	    response->answer = SUCCESS;
	    return(SUCCESS);
	}

	FD_ZERO(&readmask);

	t_abort_task = abort_task;
	abort_task = FALSE;

	for (quit = FALSE, i = 0; i < MAX_TRIES && !quit && !abort_task; i++) {
	     errno = 0;
	     timeout.tv_sec = DWAITING_TIME;
	     timeout.tv_usec = DWAITING_TIME;
	     FD_SET(sockt, &readmask);

	     cc = select(sockt + 1, &readmask, (fd_set *)0, (fd_set *)0, &timeout);
	     if (cc > 0)
		 quit = TRUE;
	     else if (errno == EINTR)
		 abort_task = TRUE;
	     else if (errno)
		 break;
	}

	if (abort_task)
	    return(TASK_ABORTED);
	abort_task = t_abort_task;

	if (!quit)
	    return (SELECT_ERROR);

	t_abort_task = abort_task;
	abort_task = FALSE;

	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);
	for (quit = FALSE, i = 0; i < MAX_TRIES && !quit && !abort_task; i++) {
	     errno = 0;
	     cc = recvfrom(sockt, (char *)response, sizeof(*response), 0, &fromaddr, &fromlen);
	     if (cc == sizeof(*response))
		 quit = TRUE;
	}
	setitimer(ITIMER_REAL, &ztimer, (struct itimerval *)0);
	if (abort_task)
	    return(TASK_ABORTED);
	abort_task = t_abort_task;

	if (quit) {
	    *addr_in = fromaddr;
	    response->id_num = ntohl(response->id_num);
	    return (0);
	}
	return (RECVFROM_ERROR);
}

int	establish_connection(msg, rp, addr)
	CTL_MSG *msg;
	CTL_RESPONSE *rp;
	struct sockaddr *addr;
{
	int sockt, fd, c;
	char *sp, *sp2;
	CTL_RESPONSE response;
	static MESSAGE_REC mesg1, mesg2;
	static MESSAGE_REC *msp = &mesg1;
	char tries, found, global;
	struct sockaddr_in temp_addr;


	if (get_sockt(SOCK_STREAM, &sockt, &temp_addr)) {
	    rp->answer = FAILED;
	    msg_out(relay_sockt, addr, sizeof(*addr), (char *)rp, sizeof(*rp), &response, addr, FALSE);
	    relay_msg(RESTART, 0);
	    return (0);
	}

	found = global = FALSE;
	if (answer_machine) {
	    if ((fd = open(MESSAGE_OUT, O_RDONLY, 0666)) >= 0) {
		while (read(fd, (char *)msp, sizeof(MESSAGE_REC)) > 0)
		   if (mesg1.status == VALID) {
		      if (!strcmp(msg->l_name, mesg1.u_name)) {
			 found = TRUE;
			 break;
		      }
		      else if (!strcmp(mesg1.u_name, ANYBODY_MSG)) {
			 mesg2 = mesg1;
			 global = TRUE;
		      }
		   }
	        close(fd);
	    }
	    if (found)
	       sp = mesg1.line;
	    else if (global)
	       sp = mesg2.line;
	    else
	       sp = xcomm_msg;
	}

	tries = 0;
	itimer.it_value.tv_sec = CONNECT_SECONDS; 
	itimer.it_interval = itimer.it_value;
	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);
	do {
		if (connect(sockt, &msg->addr, sizeof(msg->addr)) < 0) {
		    if (errno && errno != EINTR)
			tries = MAX_TRIES;
		    else
		        tries++;
		}
		else 
		    break;
	} while (tries != MAX_TRIES && !abort_task);
	setitimer(ITIMER_REAL, &ztimer, (struct itimerval *)0);

	if (tries >= MAX_TRIES)
	    relay_msg(LOST_CONNECTION, 0);
	else if (answer_machine) {

	    write(sockt, my_edchars, 3);
	    read(sockt, his_edchars, 3);

	    for (; *sp; sp++) write(sockt, sp, 1);

	    doEditing(msg, his_edchars, sockt);
	}
	else if (!abort_task)
	    send_file_d(sockt);
	close(sockt);
}

int	doEditing(msg, his_edchars, sockt)
	CTL_MSG *msg;
	char his_edchars[];
	int sockt;
{
	char *sp;
	int fd, c;
	char ch, *sp3, *sp2;
	struct sockaddr_in *ap;
	static MESSAGE_REC mesg1;

	c = 0;
	abort_task = 0;
	memset((char *)&mesg1, '\0', sizeof(MESSAGE_REC));
	for (sp = mesg1.line; sp != &mesg1.line[MESSAGE_LENGTH -1];) {
	     if (read(sockt, &ch, 1) <= 0)
		 break;
	     if (ch == QUIT_CODE || ch == 3)
		 break;
	     if (ch == his_edchars[0] || ch == his_edchars[1] || ch == his_edchars[2])
	         EditHisinput(ch, his_edchars, mesg1.line, &sp);
	     else *sp++ = ch;
	}
	*sp = '\0';

	mesg1.status = VALID;
	ap = (struct sockaddr_in *)&msg->ctl_addr;
	mesg1.address = ap->sin_addr;

	strcpy(mesg1.u_name, msg->l_name);
	gettimeofday(&mesg1.time, (struct timezone *)0);
	
	if ((fd = open(MESSAGE_IN, (O_CREAT | O_RDWR), 0666)) >= 0) {
	    int i;
	    int found;
	    MESSAGE_REC mesg2;

	    found = FALSE;
	    for (i = 0; read(fd, (char *)&mesg2, sizeof(MESSAGE_REC)) > 0; i++) 
		 if (mesg2.status != VALID) {
		     if (lseek(fd, (i * sizeof(MESSAGE_REC)), L_SET) >= 0) {
			 write(fd, (char *)&mesg1, sizeof(MESSAGE_REC));
			 found = TRUE;
			 break;
		     }
		 }
	    if (!found)
		if (lseek(fd, (i * sizeof(MESSAGE_REC)), L_SET) >= 0) {
		    write(fd, (char *)&mesg1, sizeof(MESSAGE_REC));
		    found = TRUE;
		}	
	    close(fd);
	    if (found)
		relay_msg(NEW_MESSAGE, i);
	}
	else
	    fprintf(stderr, "Error opening in-comming message file.");
}
static
void	sigterm_handler()
{
	error_exit(0);
}
static	
void	sigusr1_handler()
{
	abort_task = TRUE;
	signal(SIGUSR1, (void (*)()) sigusr1_handler);
	return;
}
static
void	sigusr2_handler()
{
	error_exit(0);
}
static
void	sigpipe_handler()
{
	abort_task = TRUE;
	signal(SIGPIPE, (void (*)()) sigpipe_handler);	
	return;
}
static
void	sigalrm_handler()
{
	signal(SIGALRM, (void (*)()) sigalrm_handler);
	return;
}
