/*
 * Copyright (c) 1992 Kent State University.
 * All rights reserved.  The Kent State University 
 * software License Agreement specifies the terms and conditions 
 * for redistribution.
 *
 *	@(#)xcomm_ads.c	1.1 (Kent State) 2/28/92
 *
 *	Author: Paul B. Ricks
 *
 *	The Gnu Licensing Agreement Applies to this software.
 */


/*
 *	xcomm_ads.c  contains routines that make up the address server for xcomm.  This 
 *	server is rather simple, but it gets the job done. When an xcomm is started, it 
 *	registers its relay address with its local daemon which in turns registers the address
 *   	with the address server. Since there is only one address server, a person can be 
 *	found on the local network simply by going through the address server. If the server is
 *	not up for some reason, then the local daemons must check there local address database.
 *	to find the relay address of a particular person. In this respect, xcomm will then
 *	perform just like the regular talk program (restricted to a host machine).
 *
 */

#ifndef lint
 static char copyright[] =
"@(#) Copyright (c) 1992 Kent State University.\n\
 All rights reserved.\n";
#endif /* not lint */

#ifndef lint
 static char sccsid[] = "@(#)xcomm_ads.c 1.1 (Kent State Univ.) 2/28/92";
#endif /* not lint */

/*

                            NO WARRANTY

   BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

                     END OF TERMS AND CONDITIONS
*/


#include "talkd.h"
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/file.h>

#define	TRUE		1
#define FALSE		0

#define MAX_TRIES	3
#define	NAME_SIZE	12
#define	ADDRESS_LEN 	81
#define	SUBNET_LEN 	9
#define	HOST_NAME_LEN 	65
#define EBUF_LEN	81
#define	RELAY_SIZE	9087
#define	LOCAL_MACHINES	"/etc/hosts"
#define REDO_AGE	40000009L
#define WAITING_TIME	60 
#define ONE_MINUTE	60
#define COUPLE_OF_SECS  5

#define	SUBNET_ADDR	"131.123.2"

#ifdef	XCOMM_NO_SU
#define RELAY_DIR	"./"
#define	RELAY_TMP	"./relay_tmp"
#define	DONT_CONTACT	"./bad_deamons"
void	syslog();
#else
#define RELAY_DIR	"/usr/local/xcomm/"
#define	RELAY_TMP	"/usr/local/xcomm/relay_tmp"
#define	DONT_CONTACT	"/usr/local/xcomm/bad_deamons"
#endif
/*
 *
	constants of interest:
	----------------------

	RELAY_SIZE	maximum size of the address database
	SUBNET_ADDR	local sub-net address
	RELAY_TMP	file used in updating address database
	DONT_CONTACT	host on local networks not to contact
	LOCAL_MACHINES	where to find address for local machines 
	REDO_AGE	the age at which if the database exceeds
			RELAY_SIZE, then the local daemons must be
			made to re-register their relay connections.
			This is intended to avoid looping because
			the new database must be this old in-order
			to have the daemons re-register.
	RELAY_FILE	file containing relay (xcomm) adrressses.
 *
 */

char RELAY_FILE[EBUF_LEN];

typedef struct {
	u_char  status;			/* status of address in database */
	u_char  phone_status;		/* status of phone line (off/on) */
	char	u_name[NAME_SIZE];	/* user name */
	char	d_addr[NAME_SIZE];	/* daemon address */
	struct  sockaddr_in c_addr;	/* xcomm relay address */
} SERVER_MSG;


CTL_MSG msg;
char debugging = 0;
u_short server_port;
char ebuf[EBUF_LEN];
struct sockaddr_in l_deamon = { AF_INET };

void sigalrm_handler(), do_nothing();

main(argc, argv)
     int	argc;
     char	**argv;
{
	int cc;
     	FILE *fp;
	int sockt;
	CTL_MSG msg;
	CTL_MSG *msp = &msg;
	int msglen = sizeof(msg);
	struct sockaddr_in fromaddr, ztest;
	struct sockaddr *fsp = (struct sockaddr *)&fromaddr;
	int fromlen = sizeof(fromaddr);
	struct itimerval itimer;

	if (argc > 1) {
	    if (!strcmp(argv[1], "-d"))
		debugging = TRUE;
	}

#ifndef XCOMM_NO_SU

        if (getuid()) {
            fprintf(stderr, "%s: getuid: not super-user\n", argv[0]);
            exit(1);
        }

        openlog("xcommads", LOG_PID, LOG_DAEMON);
#endif

	{
	    char	hostname[32];
	    if (gethostname(hostname, sizeof(hostname) - 1) < 0) {
		syslog(LOG_ERR, "gethostname: %m");
		exit(0);
	    }
	    sprintf(RELAY_FILE, "%-s%-s_SRelays", RELAY_DIR, hostname);
	}

#ifndef XCOMM_NO_SU

        if (chdir("/dev") < 0) {
            syslog(LOG_ERR, "chdir: /dev: %m");
            _exit(1);
        }
#endif



/*
 *	the following six lines should be commented out if xcomm_ads is being started by
 *	daemon starter.
 *
 */

#ifdef XCOMM_NO_SU

	if ((sockt = get_socket(SOCK_DGRAM, XCOMM_ADS_PORT)) == -1) {
	    fprintf(stderr, "Error establishing port for server\n");
	    exit(0);
	}
	dup2(sockt, 0);
	close(sockt);
#endif
	for (; ;errno = 0) {


	     signal(SIGALRM, sigalrm_handler); 
#ifdef XCOMM_NO_SU
	     itimer.it_interval.tv_sec = ONE_MINUTE * 10;
#else
	     itimer.it_interval.tv_sec = ONE_MINUTE;
#endif 
	     itimer.it_interval.tv_usec = 0;
	     itimer.it_value = itimer.it_interval;
	     setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);

	     cc = recvfrom(0, (char *)msp, msglen, 0, fsp, &fromlen);

	     itimer.it_value.tv_sec = itimer.it_value.tv_usec = 0;
	     setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);
	     signal(SIGALRM, do_nothing); 	/* need to since the signal is */
						/* used else where and the intention */
						/* then is not to terminate */

	     if (cc != msglen) {
	         if (errno != EINTR)
		     syslog(LOG_WARNING, "recvfrom: %m");
	         continue;
	     }

	     fromaddr.sin_family = ntohs(fromaddr.sin_family);
	     if (fromaddr.sin_family != AF_INET) {
	         msp->answer = BADVERSION;
	         (void)msg_out((char *)msp, msglen, fsp, fromlen, 0);
	         continue;
	     }
/*
 *	need to uncomment the else section when ready to install 
 *
 */

#ifndef XCOMM_NO_SU
	     else if (fromaddr.sin_port > IPPORT_RESERVED) {
	         msp->answer = PERMISSION_DENIED ;
	         (void)msg_out((char *)msp, msglen, fsp, fromlen, 0);
	         continue;
	     }
#endif

	     switch (msg.type) {
		case PING:	     (void)msg_out((char *)msp, msglen, fsp, fromlen, 0);
				     break;

		case ANNOUNCE:	
	        case LOOK_UP:	     perform_lookup(msp, fsp, fromlen);
				     break;

	 	case PHONELINE:	     (void)toggle_phoneline(msp);
				     break;

	        case RE_REGISTER:
	        case REGISTER:	     perform_registration(msp);
				     break;

	        case DE_REGISTER: 
	        case DE_REGISTERALL: perform_deregistration(msp);
				     break;

	        default:	     continue;
	    }
	    if (debugging)
		print_msg(msp);
	}
}


int	perform_lookup(msg, fromaddr, fromlen)
	CTL_MSG	*msg;
	struct sockaddr *fromaddr;
	int fromlen;
{
	int fd;
	int found;
	CTL_MSG rmsg;
	SERVER_MSG smsg;
	int smsglen = sizeof(smsg);
	int msglen = sizeof(*msg);


	rmsg = *msg;
	rmsg.answer = NOT_HERE;
	if ((fd = open(RELAY_FILE, O_RDWR, 0666)) < 0) {
 	    fprintf(stderr, "Couldn't access \"%s\"\n", RELAY_FILE); 
	    (void)msg_out((char *)&rmsg, sizeof(rmsg), fromaddr, fromlen, 0);
	    exit(0);
     	}

    	for (found = FALSE; !found && read(fd, (char *)&smsg, smsglen) > 0;) { 
	   if (smsg.status == NOT_VALID) continue;
	   if (!strcmp(msg->r_name, smsg.u_name)) { 
	       found = TRUE;
	       rmsg.addr = *(struct sockaddr *)&smsg.c_addr;
	       if (msg->type == LOOK_UP)
		   rmsg.answer = IS_HERE;
	       break;
	   }
	}
	close(fd);

	if (debugging) {
	    struct sockaddr_in *tp = (struct sockaddr_in *)&smsg.c_addr;

	    fprintf(stderr, "ads:: Port: %d\n", tp->sin_port);
	    fprintf(stderr, "ads:: Family: %d\n", tp->sin_family);
	    fprintf(stderr, "ads:: Address: %-s\n", inet_ntoa(tp->sin_addr.s_addr));
	    fprintf(stderr, "ads:: Callee: %-s\n", smsg.u_name);
	    fprintf(stderr, "ads:: Status of Phone line: %-s\n", smsg.phone_status != ON_LINE ?
		    "Off line":"On line");
	}

	if (found) {
	    if (msg->answer == PLEASE_RELAY && smsg.phone_status != ON_LINE)
	        rmsg.answer = PERMISSION_DENIED;
	    else if (msg->answer == PLEASE_RELAY) {
		msg->type = ANNOUNCE;
	        if (msg_out((char *)msg, msglen, (struct sockaddr *)&smsg.c_addr, sizeof(smsg.c_addr), 0) == -1) {
	            rmsg.answer = BAD_RELAYADDR;	
		    rmsg.type = LOOK_UP;
		    syslog(LOG_WARNING, "Error contacting local deamon:: %m");
	        }
	        else
	            rmsg.answer = SUCCESS;
	    }
	}
	else if (debugging)
	   fprintf(stderr, "\07\07Not found\n");
	(void)msg_out((char *)&rmsg, msglen, fromaddr, fromlen, 0);
}



int	contact_all()
{
	char **ap, *sp;
     	FILE *fp, *fp2;
	int cc, j, i = 0;
	int found = FALSE;
	int dont_call = 0;
	CTL_MSG msg; 
	CTL_MSG *mp = &msg;
	struct servent *sn;
	int msg_len = sizeof(msg);
	char buf[ADDRESS_LEN + 1];
	/*
	char ebuf[EBUF_LEN];
*/
	char hostname[HOST_NAME_LEN];
	struct sockaddr *dsp;

	gethostname(hostname, sizeof(hostname) - 1);
	if ((sn = getservbyname("xcommd", "udp")) == NULL) {
	    syslog(LOG_WARNING, "trying to get talk port. %m");
	    exit(0);
	}

     	if ((fp = fopen(LOCAL_MACHINES, "r")) == NULL) {
	    sprintf(ebuf, "Couldn't access %-s %%m", LOCAL_MACHINES); 
	    syslog(LOG_WARNING, ebuf); 
	    return 0;
	}

	msg.pid = htonl(0);
	msg.vers = TALK_VERSION;
	msg.type = RE_REGISTER;
	msg.ctl_addr.sa_family = msg.addr.sa_family = htons(AF_INET);

	l_deamon.sin_port = htons(sn->s_port);

     	if ((fp2 = fopen(DONT_CONTACT, "r")) != NULL) {
	   fgets(buf, ADDRESS_LEN, fp2);
	   dont_call = atoi(buf);
	   if ((ap = (char **) calloc((unsigned)dont_call, sizeof(char *))) == NULL)
	       dont_call = 0;
	   else {
	       while (fgets(buf, ADDRESS_LEN, fp2) != NULL && i < dont_call) {
	          if (buf[0] == '#') continue;
	          for (sp = buf; *sp && !isspace(*sp) && isprint(*sp) && *sp != '\n'; sp++);
	          if (*sp) 
		     *sp = '\0';
		  else
		     continue;
		  if ((ap[i] = (char *) calloc(strlen(sp), sizeof(char))) != NULL)
		      strcpy(ap[i++], buf);
		  else {
		      dont_call = 0;
		      while (i)
			 cfree(ap[--i]);
		      cfree(ap);
		      break;
		  }
	       }
	   }
	   fclose(fp2);
	}

	dsp = (struct sockaddr *)&l_deamon;
	while (fgets(buf, ADDRESS_LEN, fp) != NULL) {
	   if (buf[0] == '#' || strncmp(buf, SUBNET_ADDR, SUBNET_LEN)) continue;
	   for (sp = buf; *sp && !isspace(*sp) && isprint(*sp) && *sp != '\n'; sp++);
	        if (*sp) 
	           *sp = '\0';
	        else
	           continue;
	   for (found = FALSE, j = 0; !found && j < dont_call; j++)
	       if (!strcmp(buf, ap[j]))
		    found = TRUE;
	   if (found) continue;
	   if ((l_deamon.sin_addr.s_addr = inet_addr(buf)) == -1) continue;
	   if (msg_out((char *)mp, msg_len, dsp, sizeof(*dsp), 0) == -1) {
	       sprintf(ebuf, "Error contacting local deamon:: (%-s) %%m", buf);
	       syslog(LOG_WARNING, ebuf);
	   }
	}

	fclose(fp);
	if (dont_call > 0) {
	    while (dont_call)
	       cfree(ap[--dont_call]);
	    cfree(ap);
	}
}


int	perform_deregistration(msg)
	CTL_MSG	*msg;
{

	int update;
	long items;
	int fd, fd2;
	SERVER_MSG smsg;
	int smsglen = sizeof(smsg);
	int redo = FALSE;
	struct stat stbuf;

	if ((fd = open(RELAY_FILE, O_RDWR, 0666)) < 0) {
	    syslog(LOG_WARNING, "perform_deregistration: Couldnt access relay file %m");
	    return (0);
	}

	if (stat(RELAY_FILE, &stbuf) > -1) {	
	    if (stbuf.st_size > RELAY_SIZE) {
	       time_t  ct;

	       ct = time((time_t *)0);
	       if ((long)(ct - stbuf.st_ctime) > REDO_AGE)
		   redo = TRUE;
	    }
	}
	else
	    syslog(LOG_WARNING, "perform_deregistration: stat() %m");

	for (update = FALSE, items = 0; read(fd, (char *)&smsg, smsglen) > 0; items++) {
	   update = FALSE;
	   if (smsg.status == VALID) {
	       if (msg->type == DE_REGISTERALL && !strcmp(msg->l_name, smsg.d_addr)) 
	           update = TRUE;
	       else if (msg->type != DE_REGISTERALL && !strcmp(msg->l_name, smsg.u_name))
	           update = TRUE;
	   }
	   if (update) {
	       smsg.status = NOT_VALID;		/* invalidate his relay address */
	       if (lseek(fd, (smsglen * items), L_SET) >= 0)
		  write(fd, (char *)&smsg, smsglen);
	       else
		   syslog(LOG_WARNING, "perform_deregistration: fsetpos() %m");
	       if (msg->type != DE_REGISTERALL) 
	           break;
	   }
	}

	if (redo) {					
	    if ((fd2 = open(RELAY_TMP, (O_CREAT | O_TRUNC | O_WRONLY), 0666)) < 0) {
	        syslog(LOG_WARNING, "perform_deregistration: Couldnt access relay file %m");
	        syslog(LOG_WARNING, "Couldnt create tmp relay file %m");
	        close(fd);
	    }
	    else {
	       lseek(fd, 0L, L_SET);
	       for(; read(fd, (char *)&smsg, smsglen) >= 0;)
		  if (smsg.status == VALID)
		      write(fd2, (char *)&smsg, smsglen);
	       close(fd);
	       close(fd2);
	       rename(RELAY_TMP, RELAY_FILE);
	       if (stat(RELAY_FILE, &stbuf) > -1) {
	          if (stbuf.st_size > RELAY_SIZE) {	
		      unlink(RELAY_FILE);
		      contact_all();
		  }
	       }
	       else
	           syslog(LOG_WARNING, "perform_deregistration: stat() %m");
	   }
	}
	else
	    close(fd);
}



int	perform_registration(msg)
	CTL_MSG	*msg;
{
	int fd;
	int updated = FALSE;
	SERVER_MSG smsg, smsg2;
	int smsglen = sizeof(smsg);
	long items;


	if ((fd = open(RELAY_FILE, (O_CREAT | O_RDWR), 0666)) < 0) {
	    syslog(LOG_WARNING, "perform_registration: Couldnt access relay file %m");
	    return (0);
	}

	smsg2.status = VALID;
	smsg2.phone_status = msg->answer;
	smsg2.c_addr = *(struct sockaddr_in *)&msg->addr;
	strcpy(smsg2.u_name, msg->l_name);
	strcpy(smsg2.d_addr, msg->r_name);

	for (items = 0; read(fd, (char *)&smsg, sizeof(smsg)) > 0; items++) {
	   if (smsg.status == NOT_VALID && !updated) {
	       if (lseek(fd, sizeof(smsg)*items, L_SET) >= 0) {
	           updated = TRUE;
	           if (write(fd, (char *)&smsg2, sizeof(smsg2)) != sizeof(smsg2))
	               syslog(LOG_WARNING, "[0] write has problems %m");
	       }
	       else
	           syslog(LOG_WARNING, "perform_registration: [1] lseek() %m");
	   }
	   else if (smsg.status == VALID && !strcmp(msg->l_name, smsg.u_name)) {
	       if (lseek(fd, (sizeof(smsg) * items), L_SET) >= 0) {
	           if (!updated) {
	               updated = TRUE;
	               if (write(fd, (char *)&smsg2, sizeof(smsg2)) != sizeof(smsg2))
			   syslog("[1] write has problems %m");
		       syslog(LOG_WARNING, "Just updated -- overwrote existing");
	           }
	           else {
	               smsg.status = NOT_VALID;
	               if (write(fd, (char *)&smsg, sizeof(smsg)) != sizeof(smsg))
			   syslog(LOG_WARNING, "[2] write had problem %m");
		   }
	       }
	       else
	          syslog(LOG_WARNING, "perform_registration: [2] lseek() %m");
	   }
	}

	if (!updated) {
	    if (lseek(fd, sizeof(smsg)*items, L_SET) >= 0) {
	        if (write(fd, (char *)&smsg2, sizeof(smsg2)) != sizeof(smsg2))
	            syslog(LOG_WARNING, "[3] write has problems %m");
	    }
	    else
	        syslog(LOG_WARNING, "perform_registration: [3] lseek() %m");
	}
	close(fd);
}

int	get_socket(type, port)
	int	type;
	int 	port;
{
	FILE *fp;
	char *sp;
	int sockt;
	int addrlen;
	int cc, len;
	struct sockaddr_in saddress, tempaddr;
	struct sockaddr *sap = (struct sockaddr *)&saddress;
	int saplen = sizeof(saddress);
	int templen = sizeof(tempaddr);
        struct  hostent *hp;
        char my_machine_name[80];


	if ((sockt = socket(AF_INET, type, 0)) < 0) {
	    syslog(LOG_WARNING, " couldn't get socket %m");
	    return (-1);
	}

        gethostname(my_machine_name, sizeof(my_machine_name));
        if ((hp = gethostbyname(my_machine_name)) == NULL) {
            syslog(LOG_WARNING, "Could get host machine address %m");
            exit(-1);
        }

	memset((char *)sap, '\0', saplen);
	saddress.sin_port = htons(port);
	saddress.sin_family = htons(AF_INET);
        memcpy((char *)&saddress.sin_addr.s_addr, (char *)hp->h_addr, hp->h_length);

	if (bind(sockt, sap, saplen) < 0) {
	    syslog(LOG_WARNING, "bind %m");
	    close(sockt);
	    return(-1);
	}

	if ((cc = getsockname(sockt, (char *)&tempaddr, &templen)) < 0) {
	    syslog(LOG_WARNING, "Error getting name for socket %m");
	    close(sockt);
	    return(-1);
	}
	return (sockt);
}

int	toggle_phoneline(msg)
	register CTL_MSG *msg;
{
	int fd;
	long items;
	SERVER_MSG smsg;
	int smsglen = sizeof(smsg);

	
	if ((fd = open(RELAY_FILE, O_RDWR, 0666)) < 0) {
	    syslog(LOG_WARNING, "toggle_phoneline: Couldnt access relay file %m");
	    return (0);
	}

	for (items = 0; read(fd, (char *)&smsg, smsglen) > 0; items++) {
	   if (smsg.status == VALID)
	       if (!strcmp(smsg.u_name, msg->l_name)) {
                   if (debugging)
		       fprintf(stderr, "\07\07His phone has been toggled");
	           smsg.phone_status = msg->answer;
	           if (lseek(fd, (items * smsglen), L_SET) >= 0)
		       write(fd, (char *)&smsg, smsglen);
	           else
		       syslog(LOG_WARNING, "toggle_phoneline: 1 fseek() %m");
	           break;
	       }
	}
	close(fd);

	return(0);
}


int	msg_out(data, datalen, addr, addrlen, sockt)
	char *data;
	int datalen;
	struct sockaddr *addr;
	int addrlen;
	int sockt;
{
	int cc;
	int ntimes = 0;
	struct itimerval itimer;

	itimer.it_interval.tv_sec = COUPLE_OF_SECS;
	itimer.it_interval.tv_usec = 0;
	itimer.it_value = itimer.it_interval;
	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);

	errno = 0;
	do {
		cc = sendto(sockt, data, datalen, 0, addr, addrlen);
		if (cc == datalen)
		    break;
	        else if (++ntimes >= MAX_TRIES) break;
	} while (errno == EINTR || !errno);

	itimer.it_value.tv_sec = itimer.it_value.tv_usec = 0;
	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);

	if (ntimes == MAX_TRIES) {
	    syslog(LOG_WARNING, "msg_out: sendto: %m");
	    return (-1);
	}
	return (0);
}


int	print_msg(msg)
	register CTL_MSG *msg;
{
	fprintf(stderr, "Request type: ");
	switch(msg->type) {
	   case ANNOUNCE: 	fprintf(stderr, "ANNOUNCE\n");
				break;
	   case PING: 		fprintf(stderr, "PING\n");
				break;
	   case LOOK_UP: 	fprintf(stderr, "LOOK_UP\n");
				break;
	   case PHONELINE:	fprintf(stderr, "PHONELINE\n");
				fprintf(stderr, "Phoneline is to be turn [%-s]\n",
					(msg->answer == ON_LINE) ? "On":"Off");
				break;
	   case RE_REGISTER:	fprintf(stderr, "RE_REGISTER\n");
				break;
	   case REGISTER:	fprintf(stderr, "REGISTER\n");
				break;
	   case DE_REGISTER:	fprintf(stderr, "DE_REGISTER\n");
				break;
	   case DE_REGISTERALL:	fprintf(stderr, "DE_REGISTERALL\n");
				break;
	   default: 		fprintf(stderr, "Unknown -- request no. (%3d)\n", msg->type);
	}
}

void	do_nothing()
{
	signal(SIGALRM, (void (*)())do_nothing); 
}

void	sigalrm_handler()
{
	exit(0);
}

#ifdef XCOMM_NO_SU

void	syslog()
{
	;
}

#endif
