/*
 * Copyright (c) 1992 Kent State University.
 * All rights reserved.  The Kent State University 
 * software License Agreement specifies the terms and conditions 
 * for redistribution.
 *
 *	@(#)xcomm_funcs.c	1.1 (Kent State) 2/27/92
 *
 *	Author: Paul B. Ricks
 *
 *	The spirit of the Gnu Licensing Agreement Applies to this software and 
 *	Kent State University should be substituted where ever Gnu appears.
 */

/*
 *	xcomm_funcs.c contains routines that were added to the regular daemons
 *	so that xcomm could be supported.
 *
 */


/*

                            NO WARRANTY

   BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

                     END OF TERMS AND CONDITIONS
*/


#include "talkd.h"
#include <string.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <sys/file.h>

#ifndef TRUE
#define	TRUE		1
#define	FALSE		0
#endif

#define	BUFLEN		80
#define	BIG_BUFFER	1024

#ifdef XCOMM_NO_SU
#define ERROR_SHELL 	"./xcomm_mail.sh"
#define LOCAL_RELAY_DIR "./"
#define BANNED_DIR 	"./"
extern void syslog();
#else
#define ERROR_SHELL 	"/usr/local/xcomm/xcomm_mail.sh"
#define LOCAL_RELAY_DIR "/usr/local/xcomm/"
#define BANNED_DIR 	"/usr/local/xcomm/"
#endif
#define WAITING_TIME	3

#define	MAX_TRIES	3

typedef struct {		/* local relay information */
	char	msg;		/* type of request */
	short	port;
	char	status;		/* status of the relay address */
	char	u_name[NAME_SIZE]; /* user name */
	struct sockaddr_in c_addr; /* xcomm relay address */
} RELAY_REC;

struct itimerval itimer;

char LOCAL_RELAYS[BUFLEN];
char RESTRICTED_USERS[BUFLEN];

extern  int  debug;
extern 	int server_ok;
extern	char hostname[];
extern  struct hostent *hp;
extern 	int gsplen, fromlen;
extern 	int needto_reply, msglen;
extern	struct sockaddr *gsp, *fsp;
extern 	struct sockaddr_in server_addr;


int	re_register_relays(msg)
	register CTL_MSG *msg;
{
	int fd;
	int sockt;
	RELAY_REC relay_r;
	char contact_server = FALSE;
	int relayr_len = sizeof(relay_r);

	if ((fd = open(LOCAL_RELAYS, O_RDONLY, 0666)) < 0)
	    return(-1);

	if ((sockt = get_socket(SOCK_DGRAM, 0)) != -1)
	    contact_server = TRUE;

	bzero((char *)msg->r_name, NAME_SIZE);
	strncpy(msg->r_name, hostname, NAME_SIZE -1);
	for (; read(fd, (char *)&relay_r, sizeof(relay_r)) > 0; )
	     if (relay_r.status == VALID) {
		 if (contact_server) {
		     strcpy(msg->l_name, relay_r.u_name);
		     msg->answer = relay_r.msg;
		 }
	         msg_out((char *)msg, sizeof(*msg), ((contact_server) ?
						    gsp:(struct sockaddr *)&relay_r.c_addr), sockt);
	     }

	close(fd);
	if (contact_server)
	    close(sockt);
	else
	    unlink(LOCAL_RELAYS);
	return (0);		
}


int	register_relay(msg)
	register CTL_MSG *msg;
{
	int fd;
	long items;
	int error = TRUE;
	int updated = FALSE;
	RELAY_REC relay_r, relay_r2;
	int relayr_len = sizeof(relay_r);

	if (!isHeLegit(msg)) return(PERMISSION_DENIED); /* is he restricted ? */

	if (server_ok) {
	    memset((char *)msg->r_name, '\0', NAME_SIZE);
	    strncpy(msg->r_name, hostname, NAME_SIZE -1);
	    msg_out((char *)msg, sizeof(*msg), gsp, 0);
	}

	if ((fd = open_relays()) < 0)
	    return(FAILED);

	relay_r2.status = VALID;
	relay_r2.msg = msg->answer;
	relay_r2.c_addr = *(struct sockaddr_in *)&msg->addr;
	relay_r2.port = ntohs(relay_r2.c_addr.sin_port);
	strcpy(relay_r2.u_name, msg->l_name);

	for (items = 0; read(fd, (char *)&relay_r, sizeof(relay_r)) > 0; items++) {
	   if (relay_r.status == NOT_VALID && !updated) {
	       if (lseek(fd, (sizeof(relay_r2) * items), L_SET) >= 0) {
	           if (write(fd, (char *)&relay_r2, sizeof(relay_r2)) != sizeof(relay_r2))
	               syslog(LOG_WARNING, "[0] write has problems %m");
		   else
		       updated = TRUE;
	       }
	       else
	           syslog(LOG_WARNING, "register_relay: [1] lseek() %m");
	   }
	   else if (relay_r.status == VALID && !strcmp(relay_r.u_name, msg->l_name)) { 
	       msg->type = QUIT;
	       msg_out((char *)msg, sizeof(*msg), (struct sockaddr *)&relay_r.c_addr, 0);
	       if (lseek(fd, (sizeof(relay_r) * items), L_SET) >= 0) {
	           if (!updated) {
	               if (write(fd, (char *)&relay_r2, sizeof(relay_r2)) != sizeof(relay_r2))
			   syslog(LOG_WARNING, "[1] write has problems %m");
		       else
	                   updated = TRUE;
		       syslog(LOG_WARNING, "Just updated -- overwrote existing %m");
	           }
	           else {
	               relay_r.status = NOT_VALID;
	               if (write(fd, (char *)&relay_r, sizeof(relay_r)) != sizeof(relay_r))
			   syslog(LOG_WARNING, "[2] write had problem %m");
		   }
	       }
	       else
	          syslog(LOG_WARNING, "register_relay: [2] lseek() %m");
	   }
	}

	if (!updated) {
	    if (lseek(fd, (sizeof(relay_r2) * items), L_SET) >= 0) {
	        if (write(fd, (char *)&relay_r2, sizeof(relay_r2)) != sizeof(relay_r2))
	            syslog(LOG_WARNING, "[3] write has problems %m");
		else
		    updated = TRUE;
	    }
	    else
	        syslog(LOG_WARNING, "register_relay: [3] lseek() %m");
	}
	close(fd);

	(void)check_talktable(msg); /* is there a pending talk request ? */
	return((updated ? SUCCESS:FAILED));
}


int	xcomm_relay(msg, mtype)
	register CTL_MSG *msg;
	int mtype;
{
	int fd;
	int found, dfound;
	long items;
	CTL_MSG msg2;
	RELAY_REC relay_r;
	int relayr_len = sizeof(relay_r);
	struct sockaddr_in taddr;

	dfound = found = FALSE;
	taddr = *(struct sockaddr_in *)&msg->ctl_addr;
	if ((fd = open(LOCAL_RELAYS, O_RDWR, 0666)) >= 0) {
	    for (items = 0; read(fd, (char *)&relay_r, relayr_len) > 0; items++) 
	       if (relay_r.status == VALID)
		   if (!strcmp(relay_r.u_name, msg->r_name)) {
		       if (mtype == DE_REGISTER && ntohs(taddr.sin_port) == relay_r.port) {
			   if (lseek(fd, (items * relayr_len), L_SET) >= 0) {
			       relay_r.status = NOT_VALID;
			       write(fd, (char *)&relay_r, relayr_len);
			       dfound = TRUE;
		           }
		       }
		       else
	                   found = TRUE;
	               break;
	           }
	    close(fd);
	}
	if (mtype == DELETE) {
	   if (found) {
	       int sockt;

	       if ((sockt = get_socket(SOCK_DGRAM, 0)) != -1) {
	           msg_out((char *)msg, sizeof(*msg), (struct sockaddr *)&relay_r.c_addr, sockt);
	           close(sockt);
	       }
	   }
	   return(SUCCESS);
	}

	if (mtype == DE_REGISTER && !dfound) {
	    if (debug)
	        fprintf(stderr, "You don't exist to be de-registering !!!\n");
	    return (-1); 
	}

	if (!found && server_ok) {
	   int sockt;

	   if ((sockt = get_socket(SOCK_DGRAM, 0)) != -1) {
	       msg->type = mtype;
	       msg->answer = PLEASE_RELAY;
	       if (debug)
	           fprintf(stderr, "talkd: contacting server with (%-s) .\n", msg->r_name);
	       if (msg_out((char *)msg, sizeof(*msg), gsp, sockt) != -1) {
		   if (mtype == DE_REGISTER) {
		       if (debug)
	                   fprintf(stderr, "talkd: server has de-registered (%-s) .\n", msg->r_name);
		       close(sockt);
		       return(SUCCESS);
		   }
		   if (msg_in((char *)&msg2, sizeof(msg2), sockt) != -1) {
		       if (msg2.answer == COULDNT_RELAY) {
			   if (msg_out((char *)msg, sizeof(*msg), (struct sockaddr *)&msg2.ctl_addr, sockt) != -1)
			       msg2.answer = SUCCESS;
		       }
		       found = TRUE;
		   }
	       }
	       close(sockt);
	       if (debug)
	           fprintf(stderr, "Was request delivered ? %-s\n", msg2.answer == SUCCESS ? "Yes":"No");
	       if (found)
		   return(msg2.answer);
	   }
	}

	if (found && mtype != DE_REGISTER) {
	    if (relay_r.msg == ON_LINE) {
	        if (msg_out((char *)msg, sizeof(*msg), (struct sockaddr *)&relay_r.c_addr, 0) != -1)
		    return(SUCCESS);
	    }
	    else 
		return(PERMISSION_DENIED);
	}
	return (-1);
}
	


int	de_register(msg)
	register CTL_MSG *msg;
{
	int fd;
	int items;
	RELAY_REC relay_r;
	int updated = FALSE;
	int relayr_len = sizeof(relay_r);

	if (server_ok)
	    msg_out((char *)msg, sizeof(*msg), gsp, 0);

	if ((fd = open_relays()) < 0) return;

	for (items = 0; read(fd, (char *)&relay_r, relayr_len) > 0; items++) {
	   if (relay_r.status == VALID)
	       if (!strcmp(relay_r.u_name, msg->l_name)) {
	           relay_r.status = NOT_VALID;
	           if (lseek(fd, (items * relayr_len), L_SET) >= 0)
		       write(fd, (char *)&relay_r, relayr_len);
	           else
		       syslog(LOG_WARNING, "de_register: 1 lseek() %m");
	           break;
	       }
	}

	close(fd);
	return(0);
}


int	open_relays()
{
	int fd;
	extern char LOCAL_RELAYS[];

	if ((fd = open(LOCAL_RELAYS, (O_CREAT | O_RDWR), 0666)) < 0) {
	    fprintf(stderr, "Couldnt access relay file: %-s", LOCAL_RELAYS);
	    syslog(LOG_WARNING, "open_relay: Couldnt access relay file %m");
	    return (-1);
	}
	return fd;
}

int	get_socket(type, port)
	int	type;
	int 	port;
{
	FILE *fp;
	static char status = 0;
	int	sockt;
	struct sockaddr_in addr, tempaddr;
	int templen = sizeof(tempaddr);
	extern struct hostent *hp;

	if ((sockt = socket(AF_INET, type, 0)) < 0)
	    return (-1);

	bzero((char *)&addr, sizeof(addr));
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	addr.sin_family = AF_INET;

	if (bind(sockt, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
	    close(sockt);
	    return (-1);
	}

	if (getsockname(sockt, (char *)&tempaddr, &templen) < 0) {
	    syslog(LOG_WARNING, "Error getting name for socket %m");
	    close(sockt);
	    return(-1);
	}
	return (sockt);
}


int	msg_out(data, datalen, addr, sockt)
	register char *data;
	int datalen;
	register struct sockaddr *addr;
	int sockt;
{
	int cc;
	int ntimes = 0;
	struct sockaddr_in *sp = (struct sockaddr_in *)addr;

	errno = 0;
	do {
		itimer.it_value.tv_sec = WAITING_TIME;
		itimer.it_interval.tv_usec = 0;
		itimer.it_value.tv_usec = itimer.it_interval.tv_usec = 0;
		setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);

		cc = sendto(sockt, data, datalen, 0, addr, sizeof(*addr));
		if (cc == datalen)
		    break;
	        else if (++ntimes >= MAX_TRIES) break;
	} while (errno == EINTR);

	if (debug)
	    fprintf(stderr, "The status of message: %s\n", ntimes >= MAX_TRIES ? "(Not Delivered)":
		"(Delivered)");
	itimer.it_value.tv_sec =  itimer.it_interval.tv_sec = 0;
	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);

	if (ntimes == MAX_TRIES)
	    return (-1);

	return (0);
}

int	msg_in(data, datalen, sockt)
	register char *data;
	int datalen;
	int sockt;
{
	int cc;
	int ntimes = 0;

	errno = 0;
	do {
		itimer.it_value.tv_sec = WAITING_TIME * 3;
		itimer.it_interval.tv_usec = 0;
		itimer.it_value.tv_usec = itimer.it_interval.tv_usec = 0;
		setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);

		cc = recvfrom(sockt, data, datalen, 0, (struct sockaddr *)0, (int *)0);
		if (cc == datalen)
		    break;
	        else if (++ntimes >= MAX_TRIES) break;
	} while (errno == EINTR);
	if (debug)
	    fprintf(stderr, "The status of message: %s\n", ntimes >= MAX_TRIES ? "(Not Received)":
		"(Received)");
	itimer.it_value.tv_sec =  itimer.it_interval.tv_sec = 0;
	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);

	if (ntimes == MAX_TRIES)
	    return (-1);

	return (0);
}

int	toggle_phoneline(msg)
	register CTL_MSG *msg;
{
	int fd;
	long items;
	RELAY_REC relay_r;
	char found = FALSE;
	int relayr_len = sizeof(relay_r);

	
	if (server_ok)
	    msg_out((char *)msg, sizeof(*msg), gsp, 0);

	if ((fd = open_relays()) < 0) return;

	for (items = 0; read(fd, (char *)&relay_r, relayr_len) > 0; items++) {
	   if (relay_r.status == VALID)
	       if (!strcmp(relay_r.u_name, msg->l_name)) {
	           relay_r.msg = msg->answer;
		   found = TRUE;
	           if (lseek(fd, (items * relayr_len), L_SET) >= 0)
		       write(fd, (char *)&relay_r, relayr_len);
	           else
		       syslog(LOG_WARNING, "de_register: 1 lseek() %m");
	           break;
	       }
	}
	close(fd);
	return((found ? SUCCESS:FAILED));
}

int	parseArgs(argc, argv)
	int argc;
	char **argv;
{
	register int i;
	int restricted, relaySpecified;
	struct hostent *hp;

	extern char LOCAL_RELAYS[];
	extern char RESTRICTED_USERS[];

	server_ok = FALSE;
	debug = relaySpecified = restricted = FALSE;

	memset((char *)&server_addr, '\0', sizeof(server_addr));
	for (i = 1; i < argc; i++)
	    if (!debug && !strcmp(argv[i], "-d"))
		debug = TRUE;
	    else if (!server_ok && !strcmp(argv[i], "-xcads")) {
		 if (++i < argc) {
		     if ((hp = gethostbyname(argv[i])) == NULL) {
		          fprintf(stderr, "Unable to obtain address of site: <%-s>.\n", argv[i]);
		          continue;
		     }
		     memcpy((char *)&server_addr.sin_addr.s_addr, (char *)hp->h_addr,  hp->h_length);
		     server_ok = TRUE;
		 }
		 else
		    fprintf(stderr, "No relay server address specified.\n");
	    }
	    else if (!restricted && !strcmp(argv[i], "-bf")) {
		if (++i < argc) {
		    restricted = TRUE;
	            sprintf(RESTRICTED_USERS, "%-s", argv[i]);
		}
		else 
		    fprintf(stderr, "No restricted file specified.\n");

	    }
	    else if (!relaySpecified && !strcmp(argv[i], "-rf")) {
		if (++i < argc) {
	           sprintf(LOCAL_RELAYS, "%-s", argv[i]);
		   relaySpecified = TRUE;
		}
		else 
		    fprintf(stderr, "No relay file specified.\n");
	    }
	    else
		    fprintf(stderr, "Invalid argument: >%-s<\n", argv[i]);
/*
	Note: you need to find a location for these files
	-------------------------------------------------
*/
	if (!restricted)
	    sprintf(RESTRICTED_USERS, "%-s%-s_bannedUsers", BANNED_DIR, hostname);
	if (!relaySpecified)
	    sprintf(LOCAL_RELAYS, "%-s%-s_Lrelays", LOCAL_RELAY_DIR, hostname);
}

int isHeLegit(msg)
	CTL_MSG *msg;
{
	int tlen;
	int allusers;
	FILE *fp, *fopen();
	char buf[BIG_BUFFER];
	char *sp, *ap, *nextptr, *get_token();

	if ((fp = fopen(RESTRICTED_USERS, "r")) != (FILE *)0) {
	    struct sockaddr_in hisaddr;
	    
	    hisaddr = *(struct sockaddr_in *)&msg->ctl_addr;

	    ap = inet_ntoa(hisaddr.sin_addr.s_addr);
	    for (; fgets(buf, BIG_BUFFER, fp) > 0; ) {
		 if (buf[0] == '#') continue; 	/* skip comments */
		 sp = get_token(buf, &tlen, &nextptr);
		 if (!strncmp(sp, ap, tlen)) {
		    allusers = TRUE;
		    for (; (sp = get_token(nextptr, &tlen, &nextptr)); ) {
		        if (!strncmp(msg->l_name, sp, tlen))
			    break;
			allusers = FALSE;
		    }
		    if (allusers) {
			fclose(fp);
			return (0);
		    }
	    	 }
	    }
	    fclose(fp);
	}
	return (1);
}

char *get_token(sp, len, nextptr)
	char *sp;
	int *len;
	char **nextptr;
{
	char *rsp = sp;

	rsp = strtok(sp, " \t\n");
	if (rsp == NULL) return ((char *)0);
	*nextptr = strtok((char *)0, " \t");
	*len = strlen(rsp);
	return (rsp);
}
